@@ -8,7 +8,7 @@ class WalletClient(object):
     def __init__(
         self,
         url: str
-    ):
+    ) -> None:
         self.token = ""
         self.url = url
 
@@ -33,11 +33,12 @@ class WalletClient(object):
             "wallet": walletname,
             "passphrase": passphrase
         }
-        r = self._httpsession.post(self.url + "/api/v1/wallets", json=req)
+        url = "{}/api/v1/wallets".format(self.url)
+        r = self._httpsession.post(url, json=req)
         if r.status_code != 200:
             self.token = ""
         else:
-            self.token = r.json()["Data"]
+            self.token = r.json()["token"]
         return r
 
     def login(
@@ -54,10 +55,10 @@ class WalletClient(object):
             "wallet": walletname,
             "passphrase": passphrase
         }
-        r = self._httpsession.post(
-            "{}/api/v1/auth/token".format(self.url), json=req)
+        url = "{}/api/v1/auth/token".format(self.url)
+        r = self._httpsession.post(url, json=req)
         if r.status_code == 200:
-            self.token = r.json()["Data"]
+            self.token = r.json()["token"]
         else:
             self.token = ""
         return r
@@ -67,8 +68,8 @@ class WalletClient(object):
         Log out from a wallet. The token is deleted from the WalletClient
         object.
         """
-        r = self._httpsession.delete(
-            "{}/api/v1/auth/token".format(self.url), headers=self._header())
+        url = "{}/api/v1/auth/token".format(self.url)
+        r = self._httpsession.delete(url, headers=self._header())
         if r.status_code == 200:
             self.token = ""
         return r
@@ -82,29 +83,31 @@ class WalletClient(object):
 
         pubKey must be a hex-encoded string.
         """
-        return self._httpsession.get(
-            "{}/api/v1/key/{}".format(self.url, pubKey),
-            headers=self._header())
+        url = "{}/api/v1/keys/{}".format(self.url, pubKey)
+        return self._httpsession.get(url, headers=self._header())
 
     def taintkey(
         self,
-        pubKey: str
+        pubKey: str,
+        passphrase: str
     ) -> requests.Response:
         """
         Label a keypair as tainted.
 
         pubKey must be a hex-encoded string.
         """
-        return self._httpsession.put(
-            "{}/api/v1/key/{}/taint".format(self.url, pubKey),
-            headers=self._header())
+        req = {
+            "passphrase": passphrase
+        }
+        url = "{}/api/v1/keys/{}/taint".format(self.url, pubKey)
+        return self._httpsession.put(url, headers=self._header(), json=req)
 
     def listkeys(self) -> requests.Response:
         """
         List information (excluding private keys) on all keypairs.
         """
-        return self._httpsession.get(
-            "{}/api/v1/keys".format(self.url), headers=self._header())
+        url = "{}/api/v1/keys".format(self.url)
+        return self._httpsession.get(url, headers=self._header())
 
     def generatekey(
         self,
@@ -118,16 +121,15 @@ class WalletClient(object):
             "passphrase": passphrase,
             "meta": metadata
         }
-        return self._httpsession.post(
-            "{}/api/v1/keys".format(self.url), json=req,
-            headers=self._header())
+        url = "{}/api/v1/keys".format(self.url)
+        return self._httpsession.post(url, headers=self._header(), json=req)
 
     def signtx(self, tx, pubKey) -> requests.Response:
         """
         Sign a transaction.
 
         tx must be a base64-encoded string, e.g.
-        tx = base64.b64encode(someBlob).decode("ascii")
+        tx = base64.b64encode(b"someblob").decode("ascii")
 
         pubKey must be a hex-encoded string.
         """
@@ -136,6 +138,24 @@ class WalletClient(object):
             "pubKey": pubKey,
             "propagate": False
         }
-        return self._httpsession.post(
-            "{}/api/v1/messages".format(self.url), headers=self._header(),
-            json=req)
+        url = "{}/api/v1/messages".format(self.url)
+        return self._httpsession.post(url, headers=self._header(), json=req)
+
+    def updatekeymetadata(
+        self,
+        pubKey: str,
+        passphrase: str,
+        metadata: List[Dict[str, str]]
+    ) -> requests.Response:
+        """
+        Update the metadata for a keypair. Any existing metadata is removed,
+        and replaced with the supplied metadata.
+
+        pubKey must be a hex-encoded string.
+        """
+        req = {
+            "passphrase": passphrase,
+            "meta": metadata
+        }
+        url = "{}/api/v1/keys/{}/metadata".format(self.url, pubKey)
+        return self._httpsession.put(url, headers=self._header(), json=req)
